@@ -35,29 +35,30 @@ pnorm <- function (q, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)
 	if(any(pos <- !neg)) {
 	    q <- q[pos] #==> now  q >= 0
 	    prec.q <- max(.getPrec(q))
-	    two <- mpfr(2, prec.q)
-	    rt2 <- sqrt(two)
-	    rr[pos] <- if(lower.tail) {
+	    two <- mpfr(2, prec.q + 4L)
+	    Irt2 <- sqrt(mpfr(0.5, prec.q + 4L)) # 1 / sqrt(2)
+	    rr[pos] <- roundMpfr(precBits = prec.q,
+              if(lower.tail) {
 		if(log.p) {
 		    r <- q
 		    sml <- q < 0.67448975
 		    if(any(sml)) {
-			eq2 <- erf(q[sml]/rt2) ## |eq2| < 1/2 <==> |q/rt2| < 0.47693627620447
+			eq2 <- erf(q[sml]*Irt2) ## |eq2| < 1/2 <==> |q*Irt2| < 0.47693627620447
 			##                        <==>  sml   <==>   |q|   < 0.67448975019608
 			r[ sml] <- log1p(eq2) - log(two)
 		    }
 		    if(any(!sml)) {
-			ec2 <- erfc(q[!sml]/rt2) ## ==> ec2 = 1-eq2 <= 1 - 1/2 = 1/2
+			ec2 <- erfc(q[!sml]*Irt2) ## ==> ec2 = 1-eq2 <= 1 - 1/2 = 1/2
 			r[!sml] <- log1p(-0.5*ec2)
 		    }
 		    r
 		}
 		else ## !log.p
-		    (1 + erf(q/rt2))/2
+		    (1 + erf(q*Irt2))/2
 	    } else { ## upper.tail
-		r <- erfc(q/rt2) / 2
+		r <- erfc(q*Irt2) / 2
 		if(log.p) log(r) else r
-	    }
+	    })
 	}
 	rr
     } else stop("(q,mean,sd) must be numeric or \"mpfr\"")
@@ -128,7 +129,15 @@ dpois <- function (x, lambda, log = FALSE,
         if(log || useLog) {
             ## NB: For large lambda, x ~= lambda this has a *LOT* of cancellation, e.g., for
             ## --  lambda = 1e100,  prec = 256  is *NOT* sufficient !!
-            r <-  -lambda  + x*log(lambda) - lfactorial(x)
+            r <- x + 0*lambda
+            isI <- is.infinite(lambda) # & is.finite(x) # x <= lambda = +Inf
+            r[ isI] <- if(log) -Inf else 0
+            ## "else"
+            if(any(!isI)) {
+                lambda <- lambda[!isI]
+                x <- x[!isI]
+                r[!isI] <- -lambda + x*log(lambda) - lfactorial(x)
+            }
             if(log) r else exp(r)
         }
 	else exp(-lambda) * lambda^x /  factorial(x)
@@ -449,16 +458,21 @@ hypot <- function(x,y, rnd.mode = c('N','D','U','Z','A')) {
 ## The Beta(a,b)  Cumulative Probabilities are exactly computable for *integer* a,b:
 pbetaI <- function(q, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE,
 		   precBits = NULL,
-                   useRational = !log.p && !is.mpfr(q) && is.null(precBits),
+                   useRational = !log.p && !is.mpfr(q) && is.null(precBits) && int2,
                    rnd.mode = c('N','D','U','Z','A'))
 {
     stopifnot(length(shape1) == 1, length(shape2) == 1,
-	      is.whole(shape1), is.whole(shape2),
+	      (i1 <- is.whole(shape1)) | (i2 <- is.whole(shape2)),
 	      shape1 >= 0, shape2 >= 0,
 	      length(lower.tail) == 1, length(log.p) == 1,
 	      0 <= q, q <= 1, ncp == 0,
 	      is.null(precBits) ||
 	      (is.numeric(precBits) && is.whole(precBits) && precBits >= 2))
+
+    int2 <- i1 && i2 # both integer -> can use rational
+### TODO: Also have finite (but non-rational) sum if only *one* is an integer number
+### ----
+
     ## Care for too large (a,b) and "integer overflow".
     ## NB:  below have 0:(b - 1) or 0:(a - 1)
     max.ab <- 2^20
@@ -507,6 +521,12 @@ pbetaI <- function(q, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE,
     }
 }
 
+## TODO  1) add (and test) R mpfr version R's bpser() in toms708.c
+##       2) compare with bpser() from package DPQ which should have more flexible variant
+##          ~/R/Pkgs/DPQ/R/beta-fns.R  &  ~/R/Pkgs/DPQ/src/bpser.c
+pbeta_ser <- function(q, shape1, shape2, log.p=FALSE) {
+}
+
 ### MPFR version >= 3.2.0 :
    "https://www.mpfr.org/mpfr-current/mpfr.html#index-mpfr_005fgamma_005finc"
 ##
